
#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################

my $error;
my $verbose=0;
my $makebuiltin=0;
my @del;
my @builtinDefines;
my @includes = ();

$ENV{LC_MESSAGES}='C'; #to get the gcc message without any localization

sub CheckGcc
{
	my $Status;
	my $tmp = "$ENV{'ATLTGT'}/tmp_gcc_v";

        $back_lang = $ENV{'LANG'};
        $ENV{'LANG'} = "C";
	$Status=system("$compiler_name -v > \"$tmp\" 2>&1");
        $ENV{'LANG'} = $back_lang;
	if ($Status != 0)
	{
		if(open TMP, ">>$tmp") {
			print TMP "command is:\n";
			print TMP "$compiler_name -v\n";
			print TMP "----------- Environment -------------\n";
			foreach my $k (sort keys %ENV){print TMP "$k=$ENV{$k}\n";}
			close TMP;
		}
		die &PerlTranslate("GccUpdate.pl", "The compiler does not work properly. TDP autoconfiguration is not complete. The status of %1 -v is %2, more details in file %3\n", "%1 is the compiler command and %2 is the status returned, %3 is the created file", $compiler_name, $Status, $tmp);
	}
	my $found = 0;
	open(GCC,"<$tmp");
	while(<GCC>)
	{
		if (/^(.*) version (?:gcc-)?(?:egcs-)?(?:cygnus-)?((\d+\.)+\d+).*$/)
		{
			print if($verbose);
			$gcc_name = $1;
			$gcc_version = $2;
			$found = 1;
			 break;
		}
	}
	close(GCC);
	push (@del,$tmp);
	return $found;
}

sub MakeDefinesFile #($$)
{
   my ($gcc_defines,$source_extension) = @_;
   my $empty = "$INTDIR/empty.$source_extension";

   # Make an empty source file
   open(EMPTY, ">$empty") || die &PerlTranslate("GccUpdate.pl", "Cannot create %1: %2\n", "%1 is the name of the file and %2 the reason given by the Operating System", $empty, $!);
   close(EMPTY);

   # Produce the defines list
   my $cmd="$compiler_name -E -dM $empty > $gcc_defines";
   print "MakeDefinesFile: $cmd\n" if($verbose);
   #print $cmd,"\n";
   $status = system ($cmd);
   unlink ($empty);
   if ($status != 0 )
   {
      $error = &PerlTranslate("GccUpdate.pl", "could not make preprocessor predefined macros list for $source_extension");
      print $error,"\n";
      return 0;
   }
   return 1;
}

sub MakeHpp #($$)
{
   my ($hpp, $defines) = @_;
   my $tmp = "$INTDIR/newatl.hpp";

   open(OLD, "<$hpp") || ($error = &PerlTranslate("GccUpdate.pl", "cannot open %1: %2", "%1 is the name of the file and %2 the reason given by the Operating System", $hpp, $!), return 0);
   open(NEW, ">$tmp") || ($error = &PerlTranslate("GccUpdate.pl", "cannot create %1: %2", "%1 is the name of the file and %2 the reason given by the Operating System", $tmp, $!), return 0);

   my $phase = 0;
   my $depth = 0;

   while (<OLD>)
   {
      if ($phase != 1)
      {
	 print NEW;
	 if (/^\s*#\s*ifdef\s*NEED_PREPRO_DEFS(\s.*)?$/ && $phase == 0)
	 {
	    $phase = 1;
	    open(DEF, "<$defines") || ($error = &PerlTranslate("GccUpdate.pl", "cannot open %1: %2", "%1 is the name of the file and %2 the reason given by the Operating System", $defines, $!), return 0);
	    while (<DEF>) {print NEW if(/\#define/);}
	    close(DEF);
	 }
      }
      else
      {
	 if (/^\s*#\s*(if|ifdef|ifndef)\s.*$/)
	 {
	    $depth++;
	 }
	 elsif (/^\s*#\s*endif.*$/)
	 {
	    if ($depth > 0)
	    {
	       $depth--;
	    }
	    else
	    {
	       $phase = 2;
	       print NEW;
	    }
	 }
      }
   }

   close(OLD);
   close(NEW);
   print $error,"\n" if($error);
   unlink ($hpp);
   rename ($tmp, $hpp);
   print &PerlTranslate("GccUpdate.pl", "%1 updated with defines\n", "%1 is the name of the file",$hpp) if($verbose);
   return 1;
}

sub GetIncludesDirList 
{
   my $tmp = "$INTDIR/gcc_act";
   my $empty = "$INTDIR/empty.cc";
   my $tmp_o = "$INTDIR/empty.o";
   my $cmd="$compiler_name -c -v $empty -o $tmp_o 2> $tmp";
   open(EMPTY, ">$empty") || ($error =  &PerlTranslate("GccUpdate.pl", "Cannot create %1: %2\n", "%1 is the name of the file and %2 the reason given by the Operating System", $empty, $!), return 0);
   close(EMPTY);

   $back_lang = $ENV{'LANG'};
   $ENV{'LANG'} = "C";
   system ($cmd);

   $ENV{'LANG'} = $back_lang;
   unlink ($empty);
   unlink ($tmp_o) if (-f $tmp_o);
   if (! -f $tmp)
   {
      print $error,"\n" if($error);
      $error = &PerlTranslate("GccUpdate.pl", "could not execute %1 in verbose mode", "%1 is the name of the compiler", $compiler_name);
      print $error,"\n";
      return 0;
   }

   open(TMP, "<$tmp") || ($error = &PerlTranslate("GccUpdate.pl", "cannot open %1: %2", "%1 is the name of the file and %2 the reason given by the Operating System", $tmp, $!), return 0);
   $go = 0;
   my $index = 0;
   my $complete=0;
   while (<TMP>)
   {
      if ($go)
      {
	 if (/^End of search list\.$/)
	 {
	    $go = 0;
	    $complete=1;
	 }
	 elsif (/^ (.*)$/)
	 {
	 	my $IncPath=$1;
	 	$IncPath="\"$IncPath\"" if($IncPath=~/\s/);
	 	$IncPath=~s/\//\\/g if($^O=~/Win32/i);
                push(@includes,$IncPath);
		$index++;
	 }
      }
      elsif (/^#include <\.\.\.> search starts here:$/)
      {
	 $go = 1;
      }
   }
   close (TMP);
   $error= &PerlTranslate("GccUpdate.pl", "Include path not complete using:\n$cmd\n","") unless($complete);
   print $error,"\n" if($error);
   print &PerlTranslate("GccUpdate.pl", "No include path found in %1 file\n","%1 is the GNU generated file",$tmp) unless($index);
   push (@del,$tmp) if($index && ($error ne ''));
   return 0 if($error);
   return 1;
}

# This subroutine removes the back-references in a path
# e.g. /home/../gcc becomes /gcc
sub canonicalize #($)
{
   my $new = shift;
   $new=~s/\\/\//g if($^O=~/Win32/i);
   my $old;
   do {
      $old = $new;
      $new =~ s|(.*)/((?!\.\.)[^/]+)/\.\./(.*)|$1/$3|;
   } until ($new eq $old);
   $new=~s/\//\\/g if($^O=~/Win32/i);
   return $new;
}

sub MakeOpp #($)
{
   my $opp = shift;
   my $tmp = "$INTDIR/newatl.opp";

   open(OLD, "<$opp") || ($error = &PerlTranslate("GccUpdate.pl", "cannot open %1: %2", "%1 is the name of the file and %2 the reason given by the Operating System", $opp, $!), return 0);
   open(NEW, ">$tmp") || ($error = &PerlTranslate("GccUpdate.pl", "cannot create %1: %2", "%1 is the name of the file and %2 the reason given by the Operating System", $tmp, $!), return 0);

   while (<OLD>) { print NEW; }
   close(OLD);
   if($gcc_version =~ /^(\d+)\.(\d+)/)
   {
   	my $v1=$1;
   	my $v2=$2;
   	my $v3=0;
   	if($gcc_version =~ /^(\d+)\.(\d+)\.(\d+)/)
   	{
   		$v3=$3;
   	}
	printf NEW "--gnu_version=%d%02d%02d\n",$v1,$v2,$v3 if(($v1 > 3) || (($v1==3)&&($v2>2)));
	print NEW "--gen_new_specialization\n" if ($v1 >=4);
   }
   foreach $include (@includes)
   {
	print NEW "--sys_include=$include\n";
	my $resolved = canonicalize ($include);
	if ($include ne $resolved)
	{
		# We also use the canonical version of the path (the #line directives
		# may appear in one way or the other)
		print NEW "--sys_include=$resolved\n";
	}
   }
   close(NEW);
   print $error,"\n" if($error);
   unlink ($opp);
   rename ($tmp, $opp);
   return 1;
}

sub MakeDef #($$)
{
   my ($deffile, $defines) = @_;
   my $tmp = "$INTDIR/newatl.def";
   my $in_macros = 0;
   my @defineStrings;

   open(OLD, "<$deffile") || ($error = &PerlTranslate("GccUpdate.pl", "cannot open %1: %2", "%1 is the name of the file and %2 the reason given by the Operating System", $deffile, $!), return 0);
   open(NEW, ">$tmp") || ($error = &PerlTranslate("GccUpdate.pl", "cannot create %1: %2", "%1 is the name of the file and %2 the reason given by the Operating System", $tmp, $!), return 0);

   while (<OLD>)
   {
      if (/Macros/ and /start/ and /do not remove this line/)
      {
	 $in_macros = 1;
	 print NEW;
      }
      elsif ($in_macros and /end/ and /do not remove this line/)
      {
	 my $line = $_;
         open(DEF, "<$defines") || ($error = &PerlTranslate("GccUpdate.pl", "cannot open %1: %2", "%1 is the name of the file and %2 the reason given by the Operating System", $defines, $!), return 0);
	 while (<DEF>) {
		 if(/^\s*\#define\s+(\w+)\s/) {
			if (grep( /^$1$/, @defineStrings)){ chomp; $_.=" /* # duplicate # */\n";}
			else { push(@defineStrings,$1);}
			print NEW ;
		 }
	 }
	 close(DEF);
	 $in_macros = 0;
	 print NEW $line;
	 if($deffile=~ m/(atct|atus_c)\.def/){
	   if($gcc_version =~ /^(\d+)\.(\d+)/)
	   {
		my $v=$1*100+$2;
		printf NEW "#ifndef NEED_ATTOLCCP_DEFS\ntypedef int _Bool;\n#define true 1\n#define false 0\n#endif\n" if ($v >=304);
	   }
	 }
      }
      elsif ($in_macros)
      {
	 # Nothing
      }
      else
      {
	 if(/^\s*\#define\s+(\w+)\s/) {
	 	if (grep( /^$1$/, @defineStrings)) { chomp; $_.=" /* # duplicate # */\n";}
	 	else { push(@defineStrings,$1);}
	 }
	 print NEW;
      }
   }
   print NEW "\n";
   foreach $include (@includes)
   {
      print NEW "#pragma attol incl_std=$include\n";
      my $resolved = canonicalize ($include);
      print NEW "#pragma attol incl_std=$resolved\n" if ($include ne $resolved);
   }

   close (OLD);
   close (NEW);
   print $error,"\n" if($error);
   unlink ($deffile);
   rename ($tmp, $deffile);
   return 1;
}

sub GccUpdate #(@)
{
	my @argv=@_;

	$|=1;
	if (!$argv[0])
	{
   		print &PerlTranslate("GccUpdate.pl", "TestRT hpp Maker for GNU C++\n");
   		print &PerlTranslate("GccUpdate.pl", "Usage: TestRTperl GccUpdate.pl [-compiler=<compiler>] [<hpp file>] [<opp file>] [<def file>]\n");
   		print &PerlTranslate("GccUpdate.pl", "This tool updates <hpp file>, <opp file>, or <def file> according to the GNU C/C++ compiler installed on the system (if no compiler is specified, g++ will be used and must be in the search path).\n");
   		return 1;# K o
	}
	else
	{
   		@extensions = ("def", "hpp", "opp");
   		%make = ();
   		$compiler_name = "";

		foreach(@argv)
   		{
   			print "arg: $_ \n";
      			if (/^-compiler=(.*)$/)
      			{
	 			$compiler_name = $1;
	 			print "compilerName is : $compiler_name\n" if($verbose);
      			}
      			elsif (/^\s*-verbose\s*$/)
      			{
      				print "Gccupdate verbose mode set\n";
	 			$verbose = 1;
      			}
      			elsif (/^\s*-builtin\s*$/)
      			{
      				print "Gccupdate builtin detection requested\n";
	 			$makebuiltin = 1;
      			}
      			elsif (/^\s*-forceVersion=([\d\.]+)\s*$/)
      			{
      				print $_,"\n";
      				$gcc_version=$1;
      				$gcc_name=$compiler_name;
      			}
     			elsif(/^\s*-anafiles=(.*)$/)
     			{
     				push(@argv,split(/[\s;]/,$1));
     				%make=();
     			}
     			elsif(/^\s*-/)
     			{
     				$compiler_name .=' '.$_;
     			}
      			elsif (/\.?(?:.*\.)+(.*)\s*$/)
      			{
	 			my $found = 0;
	 			$ext = $1;
	 			unless(-f $_){
	 				my $ftmp=$ENV{'ATLTGT'}.'/ana/'.$_;
	 				$_=$ftmp;# if(-f $ftmp);
	 			}
	 			if (-f $_)
	 			{
					for ($i = 0; $i <= $#extensions; $i++)
					{
						if (lc($ext) eq $extensions[$i])
						{
							push @{$make{lc($ext)}}, $_;
							$found = 1;
							next;
						}
					}
					if (!$found)
					{
						print &PerlTranslate("GccUpdate.pl", "GccUpdate.pl - warning: file %1 not recognized.\n", "%1 is the name of the file", $_);
					}
				}
	 			else
	 			{
 	    				print &PerlTranslate("GccUpdate.pl", "File %1 not found\n", "%1 is the name of the file", $_);
	 			}
     			}
   		}
   		if ($compiler_name eq "")
   		{
      			print &PerlTranslate("GccUpdate.pl", "GccUpdate.pl - error: no compiler specified. Using g++...\n");
      			$compiler_name = "g++";
   		}
   		$INTDIR=".";
   		print "PWD:",GetCurrentDir(),"\n" if($verbose);
   		unless($gcc_version)
   		{
			die &PerlTranslate("GccUpdate.pl", "CheckGcc error\n") unless(CheckGcc ());
		}
      		print &PerlTranslate("GccUpdate.pl", "Compiler name:") . " $gcc_name\n";
      		print &PerlTranslate("GccUpdate.pl", "Version:") . " $gcc_version\n";
 
 		my $Merror=0;
   		if ($make{def})
   		{
      			$def_c = "defines_c.hpp";
      			unless(MakeDefinesFile($def_c,"c")){
				print &PerlTranslate("GccUpdate.pl", "Error MakeDefinesFile for C\n") ;
				$Merror=1;
      			}
   		}
   		if ($make{hpp})
   		{
      			$def_cpp = "defines_cpp.hpp";
      			unless(MakeDefinesFile($def_cpp,"cpp")){
				print &PerlTranslate("GccUpdate.pl", "Error MakeDefinesFile for C++\n") ;
				$Merror=1;
      			}
   		}
   		if ($make{def} or $make{opp} or $makebuiltin)
   		{
			unless(GetIncludesDirList()) {
				print &PerlTranslate("GccUpdate.pl", "Error GetIncludeDirList\n") ;
				$Merror=1;
      			}
   		}
   		if ($make{hpp})
   		{
      			foreach $hppfile (@{$make{hpp}})
      			{
				unless(MakeHpp($hppfile,$def_cpp)) {
					print &PerlTranslate("GccUpdate.pl", "Error MakeHpp for %1\n","",$hppfile) ;
					$Merror=1;
				} else {
					print &PerlTranslate("GccUpdate.pl", "%1 successfully updated.\n", "%1 is the name of the updated file", $hppfile) if($verbose);
					print &PerlTranslate("GccUpdate.pl", "Original file was saved to %1.\n", "%1 is the name of the backup file", "$hppfile.bak") if($verbose);
 	     			}
      			}
   		}
   		if ($make{opp})
   		{
      			foreach $oppfile (@{$make{opp}})
      			{
				unless(MakeOpp($oppfile)){
					print &PerlTranslate("GccUpdate.pl", "Error MakeOpp for %1\n","",$oppfile) ;
					$Merror=1;
				} else {
					print &PerlTranslate("GccUpdate.pl", "%1 successfully updated.\n", "%1 is the name of the updated file", $oppfile) if($verbose);
					print &PerlTranslate("GccUpdate.pl", "Original file was saved to %1.\n", "%1 is the name of the backup file", "$oppfile.bak") if($verbose);
				}
      			}
   		}
   		if ($make{def})
   		{
      			foreach $deffile (@{$make{def}})
      			{
      				my $defSrcFile=$def_c;
      				$defSrcFile=$def_cpp if($deffile=~m/cpp\.def$/);
				unless(MakeDef($deffile,$defSrcFile)){
					print &PerlTranslate("GccUpdate.pl", "Error MakeDef for %1\n","",$deffile) ;
					$Merror=1;
				} else {
		    			print &PerlTranslate("GccUpdate.pl", "%1 successfully updated.\n", "%1 is the name of the updated file", $deffile) if($verbose);
		    			print &PerlTranslate("GccUpdate.pl", "Original file was saved to %1.\n", "%1 is the name of the backup file", "$deffile.bak") if($verbose);
      				}
      			}
   		}
   		push (@del,$def_c) if ($def_c);
   		push (@del,$def_cpp) if ($def_cpp);
   		if($makebuiltin) {
   			my @files;
   			push(@files,@{$make{hpp}});
   			push(@files,@{$make{def}});
   			
   			MakeBuiltin(\@files,\@includes);
   		}
	}
	if(($error) || $Merror){
		print $error,"\n" if($error);
		print "TDP unusable.\n";
		die "TDP unusable. see log for more details\n" unless($verbose);
		return 1; # K o
	}
	unlink(@del);
	return 0; #ok
}

sub add2BuiltinList
{
my $fin = shift;
my $outopen=0;
open (IN, "<$fin") or die "Can't open $fin\n";
while (<IN>)
{
	next if(/^#/);
	chomp;
	my $line=$_;
	if(/^(.*)(__builtin[^\s\(]+)\s*\((.*)$/) {
		my $def='0';
		my $before=$1;
		my $func=$2;
		my $text=$3;
		next if($before=~m/extern\s*/);

		print ">$line, $1!$2!$3\n" if($verbose);
		unless (grep( /^$func\(/, @builtinDefines)){
		    my $openP=1;
		    while($openP){
			print ">>$line\n" if($verbose); 
			while ($text =~ s/(\([^()]*\))/_X_/g) { #convert all possible matching blocks
				print ">>>$text\n" if($debug); 
			} 
			print ">>>>$text\n" if($verbose); 
			if($text=~s/^(.*)\).*$/$1/){
				$openP=0 ;
			} else {
				chomp($line=<IN>);
				$text.=$line;
				return if($line eq '');
			}
		    }
			print ">>>>>$text\n" if($verbose); 
			#$text=~m/^([^()]*)/;
			my $c=$params='';
			my $x=1;
			foreach(split(/,/,$text)) {
				$params.=$c."p$x";
				$c=',';
				$x++;
			}
			print "#define $func($params) $def //$fin\n" if($verbose);
			push(@builtinDefines,$func."($params) $def");
		}
	} 

}
close (IN);
}

sub BuiltinParseDir
{
my $dir=shift;
    print "parsing $dir for builtin\n" if($verbose);
    if(opendir(DIR,"$dir")){     
	 push(@incdir,$dir);
	 while($f=readdir(DIR)) {
		next if($f=~/^\./);
		next if($f=~/\.gch$/);
		my $file=$dir.'/'.$f;
		if(-d $file) {
			BuiltinParseDir($file);
		} else {
			add2BuiltinList($file);
		}
	 } closedir(DIR);
    } else {
      print "builtin parsing: $dir is not an existing directory\n";
    }
}

sub MakeBuiltin
{
my ($defFiles,$incl)=@_;
    # make the builtin list from include paths
    foreach $dir (@$incl) {
	    BuiltinParseDir($dir);
    }
    foreach my $f (@$defFiles) {
        my @declared=();
        open (IN,$f) or die "unable to open the file $f\n";
        while(<IN>){
            if(/^(.*)(__builtin[^\s\(]+)\s*\(/) {
                push(@declared,$2);
#                print "$2 declared\n";
            }
        } close IN;
    	open (OUT,">>$f");
    	print OUT "\n/* generated by GccUpdate::MakeBuiltin */\n";
    	foreach (@builtinDefines) { 
    		m/(__builtin[^\s\(]+)\s*\(/;
    		my $func=$1;
    		print OUT "//" if(grep(/$func/,@declared)); 
    		print OUT "\#define $_\n" 
    	}
    	close OUT;
    }
}

1;
