#!perl -w
#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2008-2010 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################

use Win32::TieRegistry;
use File::Copy;

die "Environment variable TESTRTDIR not found. Exiting.\n\n" if(!defined($ENV{'TESTRTDIR'})); # NO TRANSLATION
$TESTRTDIR="$ENV{'TESTRTDIR'}";
$SCRIPTDIR="$TESTRTDIR/lib/scripts/";
$FILESDIR="$TESTRTDIR/targets/xml/attolcc/Rhapsody/";
require $SCRIPTDIR . "Fcts.pl";

my @RhapVERSIONS=qw( 7.4 7.2 ); #list of supported version (descending order)

sub Install()
{
	my $KeyToFind="HKEY_LOCAL_MACHINE\\SOFTWARE\\Telelogic\\Rhapsody\\";
	my $InstallDir="C:\\Program Files\\Telelogic\\Rhapsody ";
	foreach (@RhapVERSIONS) {
	    #print "find key ",$KeyToFind.$_,"\n";
	    my $Key =  $Registry->{$KeyToFind.$_};
	    if ($Key){
		  $InstallDir = $Key->{"InstallationDirectory"};
		  $InstallDir =~s,[\\\/]$,,;
		  #print "InstallationDirectory = $InstallDir\n";
		  last if(-f $InstallDir."/rhapsody.exe");
	    }

	}
	#print "InstallDir=$InstallDir\n";
	unless(-f $InstallDir."/rhapsody.exe"){
		print PerlTranslate("RhapInstall.pl", "Rhapsody installation not found.\n", "");
		return 1 
	}
	
	#copy files into Rhapsody Install
	copy($FILESDIR."TestRealTime.xml",$InstallDir."/Doc/PropertyHelp/TestRealTime.xml");
	copy($FILESDIR."TestRealTime.sbs",$InstallDir."/Share/Profiles/TestRealTime.sbs");
	copy($FILESDIR."TestRTF.bat",$InstallDir."/Share/etc/TestRTF.bat");
	copy($FILESDIR."TestRT.bat",$InstallDir."/Share/etc/TestRT.bat");
	copyEnv2TDP($FILESDIR."Env2TDP.txt",$InstallDir."/Share/etc/Env2TDP.txt");
	
	#create executables
	copy ("$TESTRTDIR/bin/intel/win32/perllauncher.exe","$TESTRTDIR/bin/intel/win32/RhapTestRT.exe");
	copy ("$TESTRTDIR/bin/intel/win32/perllauncher.exe","$TESTRTDIR/bin/intel/win32/RhapTestRTreport.exe");
	insertToolMenu($InstallDir."/Rhapsody.ini");
	checkBuildBatch($InstallDir."/Share/etc/CygwinMake.bat","\%");
	checkBuildBatch($InstallDir."/Share/etc/msmake.bat","\%");
	return 0;
}

sub copyEnv2TDP
{
	my $src=shift;
	my $dest=shift;
	my $clVer=0;
	my $tmp=$ENV{TEMP};
	$tmp.="\\" if($tmp ne '');
	print PerlTranslate("RhapInstall.pl", "Generating TDP correspondance file %1\n", "%1 is the generated file",$dest);
	system("cl 2>&1 >${tmp}TestRTRhapInstall.log");
	if(open (LOG, "${tmp}TestRTRhapInstall.log"))
	{
	    while(<LOG>){
		if(/Compiler\s+Version\s+(\d+)\./){
		    $clVer=$1 -6;
		    last;
		}
	    }
	    close LOG;
	}
	unlink("${tmp}TestRTRhapInstall.log");
	if($clVer >6){
	    if(open (IN, $src)){
		open (OU, ">$dest");
		while(<IN>){
		    if(/^Microsoft/){
			s,(ATLTGT=cvisual)6,$1$clVer,;
		    }
		}
		print OU;
		close IN;
	    }
	} else {
	    copy($src,$dest);
	}
}


sub insertToolMenu
{
    my $src=shift;
    my $found=0;
    if(-f $src){
	if(open (IN, $src)){
	    open (OU, ">$src.tmp");
	    while(<IN>){
		if(/numberOfElements=(\d+)/){
		    my $L=$1;
		    $found=1;
		    my $O=$L+1;
		    my $P=$O;
		    $O++;
		    my $Q=$O;
		    $O++;
		    my $M=$O;
		    $O++;
		    my $N=$O;
		    $O++;
		    s,$L,$O,;
		    print OU ;
		    print OU <<"END";
name$P=TestRealTime RuntimeAnalysis Enable
command$P=RhapConf.exe
arguments$P=-add "$OMROOT/Profiles/TestRealTime.sbs"
initialDir$P=
JavaMainClass$P=
JavaClassPath$P=
JavaLibPath$P=
isVisible$P=1
isMacro$P=0
isPlugin$P=0
isSynced$P=0
UsingJava$P=0
applicableTo$P=
applicableToProfile$P=
helperTriggers$P=
isPluginCommand$P=0
name$Q=TestRealTime RuntimeAnalysis Disable
command$Q=RhapConf.exe
arguments$Q=-remove "$OMROOT/Profiles/TestRealTime.sbs"
initialDir$Q=
JavaMainClass$Q=
JavaClassPath$Q=
JavaLibPath$Q=
isVisible$Q=1
isMacro$Q=0
isPlugin$Q=0
isSynced$Q=0
UsingJava$Q=0
applicableTo$Q=
applicableToProfile$Q=
helperTriggers$Q=
isPluginCommand$Q=0
name$M=TestRT results
command$M=RhapTestRTreport.exe
arguments$M=
initialDir$M=
JavaMainClass$M=
JavaClassPath$M=
JavaLibPath$M=
isVisible$M=1
isMacro$M=0
isPlugin$M=0
isSynced$M=0
UsingJava$M=0
applicableTo$M=
applicableToProfile$M=
helperTriggers$M=
isPluginCommand$M=0
name$N=TestRT clean results
command$N=RhapTestRTreport.exe
arguments$N=-clean
initialDir$N=
JavaMainClass$N=
JavaClassPath$N=
JavaLibPath$N=
isVisible$N=1
isMacro$N=0
isPlugin$N=0
isSynced$N=0
UsingJava$N=0
applicableTo$N=
applicableToProfile$N=
helperTriggers$N=
isPluginCommand$N=0
name$O=TestRT clean
command$O=RhapTestRTreport.exe
arguments$O=-cleanall
initialDir$O=
JavaMainClass$O=
JavaClassPath$O=
JavaLibPath$O=
isVisible$O=1
isMacro$O=0
isPlugin$O=0
isSynced$O=0
UsingJava$O=0
applicableTo$O=
applicableToProfile$O=
helperTriggers$O=
isPluginCommand$O=0
END
		} else {
		    print OU ;
		    if(/RhapTestRTreport/){$found=0;} #already there
		}
	    }
	    close IN;
	    close OU;
	}
	
	if($found){
	    rename("$src.tmp",$src);
	}else{
	    unlink("$src.tmp");
	}
    }
}

#check the build batch file
sub checkBuildBatch
{
my $makBatFile=shift;
my $prefix=shift;
my $modif=0;
    if(open(MAK,$makBatFile)){
	if(open(OUT,">$makBatFile.tmp")){
	    while(<MAK>){
		if(/make/ && /\Q$prefix\E1/){
		    if(/(\Q$prefix\E(\d))\s*$/){
		    	my $sp=$1;
		    	my $p=$2;
		    	my $i=9;
		    	if($p >2) {
			    while($p <$i){
				s,(\Q$sp\E),$1 $prefix$i,;
				$i--;
				$modif++;
			    }
			}
		    }
		}
		print OUT;
	    }
	    close OUT;
	    close MAK;
	    if($modif){
	        rename($makBatFile, $makBatFile.'.bak');
	        rename($makBatFile.'.tmp', $makBatFile);
	    } else {
	        unlink($makBatFile.'.tmp');
	    }
	}
    }
}

exit(Install());
1;
